;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x26cdc27241ca5a1fd368b4a9891cf16a . "./code.cg2js/cg-common-control.fasl")
(#x72ff167f47b30cd200bcb7aac15f5341 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x525fcdda4a2ee02fa0ed103b542ae561 . "./code.cg2js/cg-yes-no-string.fasl")
(#x73d555fa418167b6d42e6ca8b06a629f . "./code.cg2js/cg-clipboard.fasl")
(#x8b4f93c3fe510a28dfb6d6189113d92b . "./code.cg2js/cg-dde-utils.fasl")
(#x647fa4d25e4181abce91f4e8a2792a6a . "./code.cg2js/cg-editable-text.fasl")
(#x0376643c0c85eda204b26ac066bfc302 . "./code.cg2js/cg-text-or-combo.fasl")
(#xc8f8b19e44935c1fcbaa9025b67014ff . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x40a082bab3ff4f51b97bbd8c91dd5a3a . "./code.cg2js/cg-graphics-context.fasl")
(#x94cc2e459f4fad27c19c840218fc3638 . "./code.cg2js/cg-choose-printer.fasl")
(#x9dfc28bd88c5f359e3d33806748fd2f1 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x2fedf9082e701f14f69dc322fad0d634 . "./code.cg2js/cg-edit-in-place.fasl")
(#xf83d5c97e4bde8698bdc5f7b2e93ef72 . "./code.cg2js/cg-split-bar.fasl")
(#xa0da86e5ab16559e99826779b8fb3370 . "./code.cg2js/cg-timer.fasl")
(#x5289901db8d2b43aff9a613647436676 . "./code.cg2js/cg-progress-indicator.fasl")
(#xae772c28748061af98610da49ca7bdf8 . "./code.cg2js/cg-object-editor.fasl")
(#xf38502356f96400490b6628e86b7ad10 . "./code.cg2js/cg-fill-texture.fasl")
(#x144d9be7b0e2d4a7f8b365f5ef5079bf . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xde5e1459f3556c346fe6d2068b502e2d . "./code.cg2js/cg-os-window.fasl")
(#x7390137f6d9415d38172cb8ecff0a7a4 . "./code.cg2js/cg-color-dialog.fasl")
(#xcb5cc3dbfe96ffbe8614fe3a33abc9b9 . "./code.cg2js/cg-combo-box.fasl")
(#x0ab22b80f2bc0d229d700bc50accc6c6 . "./code.cg2js/cg-directory-dialog.fasl")
(#x174740a70f1a0ffb23a921b3ad35c3fb . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xa15a80f990b1624bb5e1a72d93cd9220 . "./code.cg2js/cg-tooltip.fasl")
(#x37592e2db0238924a9ae4d9cf4b60eb9 . "./code.cg2js/cg-lettered-menu.fasl")
(#x89ff2c4e1c811126172547e17a506b80 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x773318dd7ed94d6b3bd6acd8cbff0654 . "./code.cg2js/cg-pixmap.fasl")
(#x1f9276220dab0379b1ec95784fa97091 . "./code.cg2js/cg.fasl")
(#xc90969de757dd04f427edebe9112c312 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x9ab126a0d1710dd369499fad439176d3 . "./code.cg2js/cg-template-string.fasl")
(#xe3b9992b712353560a7eef2c7ccfe550 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x311909227beb6a880d9f8b4aa86f993f . "./code.cg2js/cg-check-box.fasl")
(#x2259a4c1dc17598992696a053a9d5a96 . "./code.cg2js/cg-toggling-widget.fasl")
(#x4babbef9c0735615179a73bd2efc1c8f . "./code.cg2js/cg-radio-button.fasl")
(#x30a824fe23d20d8b53560a8bf87a2d87 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe10daa721511bd18d98dc632163359aa . "./code.cg2js/cg-scaling-stream.fasl")
(#xed1051f2e43cf12eb7705409bea8d196 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x3d09401b4e3933ea505901a82a70d83d . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf9ff8f909690c1ca250a7c71199db120 . "./code.cg2js/cg-grid-widget.fasl")
(#xeb570bdeca4aa1459baa4d82cae1e806 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x123d574f653acdea740c7483dddd0133 . "./code.cg2js/cg-item-list.fasl")
(#x42d059f2c5a48b935425b0baf1c0faed . "./code.cg2js/cg-paren-matching.fasl")
(#x3a298ea72866dea7f3bc1c7e1b65f173 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x67e51dcd84803d41b3ade186c5f65f96 . "./code.cg2js/cg-html-dialog.fasl")
(#xb2a4c06f52bbbdf1b8b79d56febc72bd . "./code.cg2js/cg-pixmap-widget.fasl")
(#x97300a78e88f901821e54518e709cda3 . "./code.cg2js/cg-project-window.fasl")
(#xbec45d7b2c65405816aea05e2238b4cd . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x9a00b3b34bd8858be47cac8af45962f3 . "./code.cg2js/cg-toolbar.fasl")
(#xad49cc69ea59f0735cdb55eadfe5a384 . "./code.cg2js/cg-caret.fasl")
(#x659603b2713c20535f713c52c7c73347 . "./code.cg2js/cg-scroll-bar.fasl")
(#x9a7b3aa820c60c3ad4abc1b38cd3ed5b . "./code.cg2js/cg-web-browser.fasl")
(#x51352100cb2631b287f1b45b9ae7e8d9 . "./code.cg2js/cg-dropping-outline.fasl")
(#x187c7aaef394d729aac44f630a33623a . "./code.cg2js/cg-font-dialog.fasl")
(#x0cfa16e5ab8d49afeef66175287e4864 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x290542b61ccb2e0565423eba0bdaf90a . "./code.cg2js/cg-yes-no-list.fasl")
(#x0f12724cbcb75620798cf8467ef142a6 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe603531adcd74eb0cd633e63dd7e4448 . "./code.cg2js/cg-lamp.fasl")
(#x91b83e532922f5bf0a20662ed6fee2de . "./code.cg2js/cg-drawable.fasl")
(#x5ff251cb2af4620804496d69a54c53bc . "./code.cg2js/cg-choice-list.fasl")
(#x4aeb6e87c1a0a3b15c166ae3d2822bed . "./code.cg2js/cg-curve.fasl")
(#x8fbeb7d445bd46bd7ca35c94e5c5893a . "./code.cg2js/cg-base.fasl")
(#xe4de806fed19d5a07aa651b47aed8c8a . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x878e1bca97b13c51038f4f9ec5973e6a . "./code.cg2js/cg-printing.fasl")
(#x52f9c00b82328a570a22d67f7806fc84 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x1d7010f99e764b446a8d33f866a3a9cd . "./code.cg2js/cg-hotspot.fasl")
(#x1e8db83687edc4b0a12242e2d694aeea . "./code.cg2js/cg-object-editor-layout.fasl")
(#x2c8d39a70ad3cfc1a8e5963bbc5f7a01 . "./code.cg2js/cg-utility-dialog.fasl")
(#xd81c26383f9f5b3e55c27bfe0e3b67bd . "./code.cg2js/cg-shortcut-menu.fasl")
(#xa60ee1d7c7b904c09b31ab35cfeab2e3 . "./code.cg2js/cg-comtab.fasl")
(#x2498299493467ce0a92b5c4ca2d73db3 . "./code.cg2js/cg-os-widget.fasl")
(#x3a36feb364db6b32410247112a06a212 . "./code.cg2js/cg-icon.fasl")
(#x26e2f27a745c228ecc9a60fa2705ddb9 . "./code.cg2js/cg-dev.fasl")
(#x41b22a0231dd28182fae7b2c773d2ab6 . "./code.cg2js/cg-calendar.fasl")
(#x306be5c6364d8b852424d7cf0902d515 . "./code.cg2js/cg-group-box.fasl")
(#x3c77a4f9d70005569414a2e14b52c788 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xb0638d0e88290475e3c4607bdeae8cbf . "./code.cg2js/cg-bitmap-pane.fasl")
(#x8e37fc31a6e1029cc3ae13d4077ed734 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xf341afeb0c1c106bb5126d14f6876ef9 . "./code.cg2js/cg-nodes.fasl")
(#x78b8da6bb5d8361941141f34c57a7302 . "./code.cg2js/cg-intersect.fasl")
(#xb42214aa088dba5f45fbd52d58eef1d1 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x5c9de8ed3dd7a72aebfa05aa561940c4 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xdf0c139dbaad8025f4da4c297de49554 . "./code.cg2js/cg-class-support.fasl")
(#xf980b7b737a9458c965d527655f8f2b2 . "./code.cg2js/cg-dialog-item.fasl")
(#x024ad9e341f12b3f1c5267dba8cd6df6 . "./code.cg2js/cg-plot-widget.fasl")
(#x3488aa1ccebf90a1f24a8fd26813152f . "./code.cg2js/cg-pixmap-printing.fasl")
(#x7b93f183d36d932e77b3f590816a0a60 . "./code.cg2js/cg-up-down-control.fasl")
(#x359e9210cafbdb768a0a23c901cb6053 . "./code.cg2js/cg-rich-edit.fasl")
(#x257e3e361d83f4b5beb23b53d5696db1 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x1eb1144be3e51beebc228fb32d2ee14b . "./code.cg2js/cg-status-bar.fasl")
(#x8451943965cf9876a094cf5879c9a14b . "./code.cg2js/cg-class-grid.fasl")
(#x4422727df647ba60634ddba119c36044 . "./code.cg2js/cg-wrap-string.fasl")
(#x6a3efe987564de26aad545d99c5fea40 . "./code.cg2js/cg-layout.fasl")
(#x274fb28533e3c2fe3cbe525947a10c4a . "./code.cg2js/cg-lisp-widget.fasl")
(#x7e141e10df214d59e0b3a9d148a2d53e . "./code.cg2js/cg-button.fasl")
(#x2d5551845764acdc5c6f57b8bd421afa . "./code.cg2js/cg-string-dialog.fasl")
(#x503dbc331b17ebda071c8defb7fa5199 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x7c943b9eab6156a9332fafdefa55f251 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xbbe9c77d8416e2ae9780e13e2dad915a . "./code.cg2js/cg-bitmap-stream.fasl")
(#xf3f6923c0092d2bddf4d81614175e2e1 . "./code.cg2js/cg-static-text.fasl")
(#x486ee0e1b12c8f8520efc9b2c10d9dc6 . "./code.cg2js/cg-outline.fasl")
(#xabbe89d043549ca5c4c932603f2e788b . "./code.cg2js/cg-gesture-emulation.fasl")
(#xb01b987540fea9f96685284cf33b93c0 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x381bc9c2fbf7c9a96981ce6e2db152ca . "./code.cg2js/cg-picture-widget.fasl")
(#xa71b5ad7ed24528c99899d78874e4078 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x4e6974d36b932827846c323c90312562 . "./code.cg2js/cg-chart-widget.fasl")
(#xcb4d054ec4c0e24613970069d88b03de . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xf3423fa75d53d854f8aaf34fb1007444 . "./code.cg2js/cg-lisp-text.fasl")
(#xb1ec9e4622d3d7d4eaaada45400301f3 . "./code.cg2js/cg-palette.fasl")
(#x02477a26ce1045164bf250924a4f1176 . "./code.cg2js/cg-property.fasl")
(#x1c28aa69c4b7b6a29e73481c444d8f67 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x9f6f3cc93c890b5e68253d37ca801f1d . "./code.cg2js/cg-file-dialog.fasl")
(#x4900a33b87ceb01b5052f66fb6e4bb1c . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xf359c2a8a5e4095c878682800f6faedf . "./code.cg2js/cg-get-pixmap.fasl")
(#x036feb57939880db4262079b04a7f28b . "./code.cg2js/cg-tray.fasl")
(#x0afe864be1eeb043ad1380e03b40ee4e . "./code.cg2js/cg-acache.fasl")
(#xf268430026c6a6bb4e622a9e7899f2fc . "./code.cg2js/cg-message-dialog.fasl")
(#xf51ffcf3a2d956f6fc969a4fc834e62e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x5eac634c746a6c70870c586f7aaa5463 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xfcfed9ee5a33f0bc45b983fb1f8ca39c . "./code.cg2js/cg-tab-control.fasl")
(#xdd1b67f527ae77b0f7d8a2ddc6785353 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6e23554a626f5ac1b527809bf6a33319 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x388a231b52545cbbd719f846a6beb415 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x351adf8d53dfea25934f5a2d2e2b0cf2 . "./code.cg2js/cg-menu.fasl")
(#x7cb24176a6ed82df5b33de28cc2f3dce . "./code.cg2js/cg-get-position.fasl")
(#x2f19f4ae24b3e722620c8cb72dba546f . "./code.cg2js/cg-trackbar.fasl")
(#x6adc3d773e3ce7807a7d09dae880d950 . "./code.cg2js/cg-selected-object.fasl")
(#x3be444f0b7c991d419b39203826c5e26 . "./code.cg2js/cg-list-view.fasl")
(#xbe88fc8b1cae0550d8e83965921dda7d . "./code.cg2js/cg-html-widget.fasl")
(#xcf81f3dd3fa57f07787cc6712a8a47e9 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x6bcb0c13e60c0533dac622c0a55ab156 . "./code.cg2js/cg-text-widget.fasl")
(#x7e3358bf072c870922578db5c00e25c3 . "./code.idejs/ide-base.fasl")
(#xd8fb60d12ff441d288e7c3b61589f343 . "./code.idejs/ide-process-browser.fasl")
(#xbed225da5c0d7050546f56a3b3ceb3f2 . "./code.idejs/ide-patch-dialog.fasl")
(#x0ea6bd396668c49641b917a1871a29d0 . "./code.idejs/ide-keyshort.fasl")
(#x64161d273a967d31ab62573ad0554810 . "./code.idejs/ide-window-list.fasl")
(#x77a16ab46613bc7f1ac97783b3cbd888 . "./code.idejs/ide.fasl")
(#x7c1b16b77ff5e43291822d34f45b9929 . "./code.idejs/ide-editor.fasl")
(#x2895959077030fcf89726d1caf2e2149 . "./code.idejs/ide-clipboard.fasl")
(#xb5f11cf57ce0af53a81e5d47ed2155d2 . "./code.idejs/ide-trace.fasl")
(#x1411b7ca33f77ee952c4d99cda128ae6 . "./code.idejs/ide-class-browser.fasl")
(#x10523da52d8746b0393dadee694760e2 . "./code.idejs/ide-grapher.fasl")
(#x270fa97e0314c280b2f80f1b1bf70477 . "./code.idejs/ide-apropos.fasl")
(#x639f01b5768dcb885bd4dc00d56a78f1 . "./code.idejs/ide-navigator.fasl")
(#x5abb5b8abdca2afac998edd96d70b545 . "./code.idejs/ide-find-definitions.fasl")
(#x9efbdcc9f145cda6f9ff639749103cb3 . "./code.idejs/ide-project.fasl")
(#x5bacac429f666200fda1a91fc38d1762 . "./code.idejs/ide-class-grapher.fasl")
(#xd5f0d893837b69cb7fdd4ad0f86e420f . "./code.idejs/ide-acache.fasl")
(#x180d5bdd7ff47f5b4164e11b163a9337 . "./code.idejs/ide-find-in-files.fasl")
(#x58410e8b64ac128be24eb077a50277d6 . "./code.idejs/ide-inspector.fasl")
(#x20492abb6afc12109037de039ae42b0e . "./code.idejs/ide-intro.fasl")
(#x79bf33fb0930983b077a09c2ad3fc2d4 . "./code.idejs/ide-package-list.fasl")
(#x7a77166adab948f7f26b894b36fae374 . "./code.idejs/ide-dev.fasl")
(#xe6319df3997fd6ca0a96b1f8e52161a6 . "./code.idejs/ide-form.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
